@@ -98,7 +98,7 @@ Application::Application (int argc, char** argv)
 		
 	main_thread = g_thread_self();
 	exe_path	= argv[0];
-		
+			
 	gtk_init_add(on_init, this);
 	guint check_error_messages_timeout_id = gdk_threads_add_timeout (1000, on_check_error_messages_timer, this);
 	gtk_main();
@@ -191,8 +191,9 @@ void Application::init()
 	g_free(exe_dir);
 	
 	log_file = new IO::Channel::Channel(application_directory + "/me-tv.log", O_CREAT | O_TRUNC | O_WRONLY);
-	
+
 	Log::write("Me TV Version: %s", PACKAGE_VERSION);
+	Log::write("Timezone: %d", timezone);
 
 	String configuration_file;
 	if (configuration_file_argument != NULL)
@@ -1095,7 +1096,99 @@ const Transponder& Application::get_current_transponder()
 {
 	return get_video_channel().get_transponder();
 }
+
+class EITDemuxers
+{
+private:
+	GSList* eit_demuxers;
+	guint demuxer_count;
+	String demuxer_path;
+
+public:
+	EITDemuxers(const String& path)
+	{
+		demuxer_path = path;
+		demuxer_count = 0;
+		eit_demuxers = NULL;
+	}
+	
+	~EITDemuxers()
+	{
+		delete_all();
+	}
 		
+	DvbDemuxer* add()
+	{
+		DvbDemuxer* demuxer = new DvbDemuxer(demuxer_path);
+		eit_demuxers = g_slist_append(eit_demuxers, demuxer);
+		demuxer_count++;
+		return demuxer;
+	}
+
+	void delete_all()
+	{
+		while (eit_demuxers != NULL)
+		{
+			delete (DvbDemuxer*)eit_demuxers->data;
+			eit_demuxers = g_slist_delete_link(eit_demuxers, eit_demuxers);
+		}
+		demuxer_count = 0;
+	}
+
+	void get_next_eit(DvbSectionParser& parser, EventInformationSection& section, gboolean is_atsc)
+	{
+		if (eit_demuxers == NULL)
+		{
+			throw Exception("No demuxers");
+		}
+		
+		DvbDemuxer* selected_eit_demuxer = NULL;
+		
+		struct pollfd fds[demuxer_count];
+		guint count = 0;
+		
+		GSList* eit_demuxer = eit_demuxers;
+		while (eit_demuxer != NULL)
+		{				
+			fds[count].fd = ((DvbDemuxer*)eit_demuxer->data)->get_fd();
+			fds[count].events = POLLIN;
+			count++;
+			eit_demuxer = g_slist_next(eit_demuxer);
+		}
+
+		guint result = ::poll(fds, demuxer_count, 5000);
+		if (result < 0)
+		{
+			throw SystemException ("Failed to poll EIT demuxers");
+		}
+		
+		eit_demuxer = eit_demuxers;
+		while (eit_demuxer != NULL && selected_eit_demuxer == NULL)
+		{
+			DvbDemuxer* current = (DvbDemuxer*)eit_demuxer->data;
+			if (current->poll(1))
+			{
+				selected_eit_demuxer = current;
+			}
+			eit_demuxer = g_slist_next(eit_demuxer);				
+		}
+
+		if (selected_eit_demuxer == NULL)
+		{
+			throw Exception("Failed to get an EIT demuxer with events");
+		}
+		
+		if (is_atsc)
+		{
+			parser.parse_psip_eis(*selected_eit_demuxer, section);
+		}
+		else
+		{
+			parser.parse_eis(*selected_eit_demuxer, section);
+		}
+	}
+};
+
 gpointer Application::epg_thread_function(Application* application)
 {
 	TRY;
@@ -1104,17 +1197,32 @@ gpointer Application::epg_thread_function(Application* application)
 	gsize stored_events_count = 0;
 	
 	const Transponder& transponder = application->get_current_transponder();
-	DvbDemuxer demuxer_eit(application->demux_path);
+	EITDemuxers demuxers(application->demux_path);
+	DvbSectionParser parser;
+	MasterGuideTable master_guide_table;
+
 	gboolean is_atsc = application->tuner->get_frontend_type() == FE_ATSC;
 	if (is_atsc)
 	{
-		demuxer_eit.set_filter(0x1FFB, EIT_ID, 0);
+		DvbDemuxer demuxer_mgt(application->demux_path);
+		demuxer_mgt.set_filter(PSIP_PID, MGT_ID, 0xFF);
+		parser.parse_psip_mgt(demuxer_mgt, master_guide_table);
+		
+		gsize size = master_guide_table.tables.size();
+		for (guint i = 0; i < size; i++)
+		{
+			MasterGuideTableTable mgtt = master_guide_table.tables[i];
+			if (mgtt.type >= 0100 && mgtt.type <= 0x017F)
+			{		
+				demuxers.add()->set_filter(mgtt.pid, PSIP_EIT_ID, 0);
+				Log::write("Set up PID 0x%02X for events", mgtt.pid);
+			}
+		}
 	}
 	else
 	{
-		demuxer_eit.set_filter(EIT_PID, EIT_ID, 0);
+		demuxers.add()->set_filter(EIT_PID, EIT_ID, 0);
 	}
-	DvbSectionParser parser;
 		
 	gboolean done = application->terminate_threads;
 	while (!done)
@@ -1122,14 +1230,9 @@ gpointer Application::epg_thread_function(Application* application)
 		try
 		{
 			EventInformationSection section;
-
-			parser.parse_eis(demuxer_eit, section);
 			
-			if (is_atsc)
-			{
-				Log::write("EIT read");
-			}
-	
+			demuxers.get_next_eit(parser, section, is_atsc);
+
 			guint service_id = section.service_id;
 			guint frequency = transponder.get_frequency();
 			
