@@ -18,14 +18,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
  
-#include "channel.h"
+#include "channel.hh"
 
 Channel::Channel(Transponder& transponder) : transponder(transponder)
 {
 	service_id = 0;
 	default_video_pid = 0;
 	default_audio_pid = 0;
-	default_subtitle_pid = 0;
 }
 
 gboolean Channel::operator ==(const Channel& channel) const
