@@ -24,6 +24,8 @@
 #include "io.hh"
 #include "application.hh"
 
+#define MAX_CHANNELS 2000
+
 static const DvbParameter inversion_list[] =
 {
 	{ "INVERSION_OFF", INVERSION_OFF },
@@ -149,10 +151,16 @@ void ChannelManager::load(int frontend_type, const String& path)
 		throw Exception(_("Me TV cannot use a channels.conf file that is not UTF-8 encoded."));
 	}
 		
-	gchar** lines = g_strsplit ( contents, "\n", 1000 );
+	gchar** lines = g_strsplit ( contents, "\n", MAX_CHANNELS );
 	int line_index = 0;
 	while ( lines[line_index] != NULL )
-	{
+	{		
+		if (line_index + 1 >= MAX_CHANNELS)
+		{
+			Log::write("Me TV has stopped processing the channels.conf file because it is currently limited to %d channels", MAX_CHANNELS);
+			break;
+		}
+
 		StringSplitter parts(lines[line_index], ":", 100);
 		line_index++;
 
@@ -281,9 +289,11 @@ void ChannelManager::load(int frontend_type, const String& path)
 			break;
 		}
 
-		transponder->add_channel(channel);		
+		transponder->add_channel(channel);
 		g_hash_table_insert(channels, (gpointer)channel->name.c_str(), (gpointer)channel);
 		channels_by_order = g_slist_append(channels_by_order, channel);
+		
+		Log::write("Loaded channel '%s'", channel->name.c_str());
 	}
 	g_strfreev ( lines );
 	
