@@ -44,7 +44,8 @@ DvbDemuxer::~DvbDemuxer()
 	if (fd != -1)
 	{
 		stop();
-		close(fd);
+		::close(fd);
+		fd = -1;
 	}
 }
 
@@ -94,20 +95,12 @@ void DvbDemuxer::set_buffer_size(unsigned int buffer_size)
 
 gint DvbDemuxer::read(unsigned char* buffer, size_t length)
 {
-	gint bytes_read = 0;
-	
-	int poll_result = poll(pfd, 1, 5000);
-	if (poll_result<0)
-	{
-		throw SystemException(_("Failed to poll for demuxer"));
-	}
-	
-	if (poll_result == 0)
+	if (!poll(5000))
 	{
 		throw TimeoutException(_("Timeout while polling for demuxer"));
 	}
-		
-	bytes_read = ::read(fd, buffer, length);
+
+	guint bytes_read = ::read(fd, buffer, length);
 	if (bytes_read == -1)
 	{
 		throw SystemException(_("Failed to read data from demuxer"));
@@ -128,3 +121,15 @@ int DvbDemuxer::get_fd() const
 {
 	return fd;
 }
+
+gboolean DvbDemuxer::poll(guint timeout)
+{
+	guint result = ::poll(pfd, 1, timeout);
+	
+	if (result == -1)
+	{
+		throw SystemException ("Failed to poll");
+	}
+	
+	return result > 0;
+}
