@@ -22,10 +22,29 @@
 #include "log.hh"
 #include "string_utility.hh"
 #include "application.hh"
+#include "libucsi/atsc/types.h"
 
 #define SHORT_EVENT			0x4D
 #define EXTENDED_EVENT		0x4E
 
+#define GPS_EPOCH			315964800
+
+void dump(guchar* buffer, gsize length)
+{
+	for (guint i = 0; i < length; i++)
+	{
+		guchar ch = buffer[i];
+		if (g_ascii_isalnum(ch))
+		{
+			Log::write ("buffer[%d] = 0x%02X; // (%c)", i, ch, ch);
+		}
+		else
+		{
+			Log::write ("buffer[%d] = 0x%02X;", i, ch);
+		}
+	}
+}
+
 static guint32 crc_table[256] = {
     0x00000000, 0x04c11db7, 0x09823b6e, 0x0d4326d9, 0x130476dc, 0x17c56b6b,
     0x1a864db2, 0x1e475005, 0x2608edb8, 0x22c9f00f, 0x2f8ad6d6, 0x2b4bcb61,
@@ -88,23 +107,10 @@ static guint32 crc32(const char *data, int len)
 Event::Event()
 {
 	event_id = 0;
-	start_time_MJD = 0;
-	start_time_UTC = 0;
+	start_time = 0;
 	duration = 0;
 	running_status = 0;
 	free_CA_mode = 0;
-	
-	dur_hour = 0;
-	dur_min = 0;
-	dur_sec = 0;
-	
-	start_year = 0;
-	start_month = 0;
-	start_day = 0;
-	
-	start_hour = 0;
-	start_min = 0;
-	start_sec = 0;
 }
 
 DvbSectionParser::DvbSectionParser()
@@ -156,6 +162,32 @@ void DvbSectionParser::parse_pas(DvbDemuxer& demuxer, ProgramAssociationSection&
 	}
 }
 
+gsize get_atsc_text(String& string, const guchar* buffer)
+{
+	size_t text_position = 0;
+	gsize offset = 0;
+	
+	gsize number_strings = buffer[offset++];
+	for (guint i = 0; i < number_strings; i++)
+	{
+		offset += 3;
+		gsize number_segments = buffer[offset++];
+		for (guint j = 0; j < number_segments; j++)
+		{
+			struct atsc_text_string_segment* segment = (struct atsc_text_string_segment*)&buffer[offset];
+			
+			uint8_t* text = NULL;
+			size_t text_length = 0;
+			
+			atsc_text_segment_decode(segment, &text, &text_length, &text_position);
+
+			string.append((const gchar*)text, text_position);
+		}
+	}
+	
+	return offset;
+}
+
 gboolean DvbSectionParser::find_descriptor(uint8_t tag, const unsigned char *buf, int descriptors_loop_len, 
 	const unsigned char **desc, int *desc_len)
 {
@@ -332,6 +364,61 @@ void DvbSectionParser::parse_pms(DvbDemuxer& demuxer, ProgramMapSection& section
 	}
 }
 
+void DvbSectionParser::parse_psip_mgt(DvbDemuxer& demuxer, MasterGuideTable& table)
+{
+	gsize section_length = read_section(demuxer);
+	guint offset = 9;
+	guint tables_defined = get_bits(&buffer[offset], 0, 16);
+	offset += 2;
+	
+	for (guint i = 0; i < tables_defined; i++)
+	{
+		MasterGuideTableTable mgtt;
+		mgtt.type = get_bits(&buffer[offset], 0, 16);
+		offset += 2;
+		mgtt.pid = get_bits(&buffer[offset], 3, 13);
+		table.tables.push_back(mgtt);	
+		offset += 7;
+		guint table_type_descriptors_length = get_bits(&buffer[offset], 4, 12);
+		offset += table_type_descriptors_length + 2;
+	}
+}
+
+void DvbSectionParser::parse_psip_eis(DvbDemuxer& demuxer, EventInformationSection& section)
+{
+	gsize section_length = read_section(demuxer);
+	guint offset = 3;
+
+	section.service_id = get_bits(&buffer[offset], 0, 16);
+	
+	offset += 6;
+	guint num_events_in_section = buffer[offset++];
+	
+	for (int i = 0; i < num_events_in_section; i++)
+	{
+		Event event;
+
+		event.event_id		= get_bits (&buffer[offset], 2, 14); offset += 2;
+		event.start_time	= get_bits (&buffer[offset], 0, 32); offset += 4;
+		event.duration		= get_bits (&buffer[offset], 4, 20); offset += 3;
+
+		event.start_time += GPS_EPOCH;
+		event.start_time += timezone;
+		
+		gsize title_length = buffer[offset++];
+		if (title_length > 0)
+		{
+			get_atsc_text (event.title, &buffer[offset++]);
+			offset += title_length;
+		}
+		
+		guint descriptors_length = buffer[offset++];
+		offset += descriptors_length;
+		
+		section.events.push_back(event);
+	}
+}
+
 void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& section)
 {
 	gsize section_length = read_section(demuxer);
@@ -354,48 +441,70 @@ void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& s
 	{
 		Event event;
 
-		event.event_id                   = get_bits (&buffer[offset], 0, 16);
-		event.start_time_MJD             = get_bits (&buffer[offset], 16, 16);
-		event.start_time_UTC             = get_bits (&buffer[offset], 32, 24);
-		event.duration                   = get_bits (&buffer[offset], 56, 24);
-		event.running_status             = get_bits (&buffer[offset], 80, 3);
-		event.free_CA_mode               = get_bits (&buffer[offset], 83, 1);
+		gulong	start_time_MJD;  // 16
+		gulong	start_time_UTC;  // 24
+		gulong	duration;
+
+		event.event_id			= get_bits (&buffer[offset], 0, 16);
+		start_time_MJD			= get_bits (&buffer[offset], 16, 16);
+		start_time_UTC			= get_bits (&buffer[offset], 32, 24);
+		duration				= get_bits (&buffer[offset], 56, 24);
+		event.running_status	= get_bits (&buffer[offset], 80, 3);
+		event.free_CA_mode		= get_bits (&buffer[offset], 83, 1);
 
 		unsigned int descriptors_loop_length  = get_bits (&buffer[offset], 84, 12);
 		offset += 12;
 		unsigned int end_descriptor_offset = descriptors_loop_length + offset;
 
-		event.dur_hour = ((event.duration >> 20)&0xf)*10+((event.duration >> 16)&0xf);
-		event.dur_min =  ((event.duration >> 12)&0xf)*10+((event.duration >>  8)&0xf);
-		event.dur_sec =  ((event.duration >>  4)&0xf)*10+((event.duration      )&0xf);
+		guint event_dur_hour = ((duration >> 20)&0xf)*10+((duration >> 16)&0xf);
+		guint event_dur_min =  ((duration >> 12)&0xf)*10+((duration >>  8)&0xf);
+		guint event_dur_sec =  ((duration >>  4)&0xf)*10+((duration      )&0xf);
 
-		if ( event.start_time_MJD > 0 )
+		event.duration = (event_dur_hour*60 + event_dur_min) * 60;
+		
+		guint event_start_day = 0;
+		guint event_start_month = 0;
+		guint event_start_year = 0;
+		
+		if (start_time_MJD > 0 && event.event_id != 0)
 		{
-			long year =  (long) ((event.start_time_MJD  - 15078.2) / 365.25);
-			long month =  (long) ((event.start_time_MJD - 14956.1 - (long)(year * 365.25) ) / 30.6001);
-			event.start_day =  (long) (event.start_time_MJD - 14956 - (long)(year * 365.25) - (long)(month * 30.6001));
+			long year =  (long) ((start_time_MJD  - 15078.2) / 365.25);
+			long month =  (long) ((start_time_MJD - 14956.1 - (long)(year * 365.25) ) / 30.6001);
+			event_start_day =  (long) (start_time_MJD - 14956 - (long)(year * 365.25) - (long)(month * 30.6001));
 
 			long k = (month == 14 || month == 15) ? 1 : 0;
 
-			event.start_year = year + k + 1900;
-			event.start_month = month - 1 - k * 12;
-		}
+			event_start_year = year + k + 1900;
+			event_start_month = month - 1 - k * 12;
 
-		event.start_hour = ((event.start_time_UTC >> 20)&0xf)*10+((event.start_time_UTC >> 16)&0xf);
-		event.start_min =  ((event.start_time_UTC >> 12)&0xf)*10+((event.start_time_UTC >>  8)&0xf);
-		event.start_sec =  ((event.start_time_UTC >>  4)&0xf)*10+((event.start_time_UTC      )&0xf);
+			guint event_start_hour = ((start_time_UTC >> 20)&0xf)*10+((start_time_UTC >> 16)&0xf);
+			guint event_start_min =  ((start_time_UTC >> 12)&0xf)*10+((start_time_UTC >>  8)&0xf);
+			guint event_start_sec =  ((start_time_UTC >>  4)&0xf)*10+((start_time_UTC      )&0xf);
 
-		while (offset < end_descriptor_offset)
-		{
-			offset += decode_short_event(&buffer[offset], event);
-		}
+			struct tm t;
+
+			memset(&t, 0, sizeof(struct tm));
+
+			t.tm_min	= event_start_min;
+			t.tm_hour	= event_start_hour;
+			t.tm_mday	= event_start_day;
+			t.tm_mon	= event_start_month - 1;
+			t.tm_year	= event_start_year - 1900;
+			
+			event.start_time = mktime(&t);
 		
-		if (offset > end_descriptor_offset)
-		{
-			throw Exception("ASSERT: offset > end_descriptor_offset");
-		}
+			while (offset < end_descriptor_offset)
+			{
+				offset += decode_event_descriptor(&buffer[offset], event);
+			}
 
-		section.events.push_back( event );
+			if (offset > end_descriptor_offset)
+			{
+				throw Exception("ASSERT: offset > end_descriptor_offset");
+			}
+
+			section.events.push_back( event );
+		}
 	}
 	section.crc = get_bits (&buffer[offset], 0, 32);
 	offset += 4;
@@ -406,27 +515,57 @@ void DvbSectionParser::parse_eis(DvbDemuxer& demuxer, EventInformationSection& s
 	}
 }
 
-gsize DvbSectionParser::decode_short_event (const guchar* buffer, Event& event)
+gsize DvbSectionParser::decode_event_descriptor (const guchar* event_buffer, Event& event)
 {
-	if (buffer[1] == 0)
+	if (event_buffer[1] == 0)
 	{
 		return 2;
 	}
-		
-	gsize descriptor_length = buffer[1] + 2;
+
+	gsize descriptor_length = event_buffer[1] + 2;
 	
-	switch(buffer[0])
-	{
+	guint descriptor_tag = event_buffer[0];	
+	switch(descriptor_tag)
+	{		
+	case EXTENDED_EVENT:
+		{
+			String language;
+			String title;
+			String description;
+
+			language = get_lang_desc (event_buffer);
+			unsigned int offset = 6;
+			guint length_of_items = event_buffer[offset];
+			offset++;
+			while (length_of_items > offset - 7)
+			{
+				offset += get_text(description, &event_buffer[offset]);			
+				offset += get_text(title, &event_buffer[offset]);
+			}
+			offset += get_text(description, &event_buffer[offset]);
+			
+			if (event.description.is_empty())
+			{
+				event.description = description;
+			}
+
+			if (event.title.is_empty())
+			{
+				event.title = title;
+			}
+		}
+		break;
+
 	case SHORT_EVENT:
 		{
 			String language;
 			String title;
 			String description;
 			
-			language = get_lang_desc (buffer);
+			language = get_lang_desc (event_buffer);
 			unsigned int offset = 5;
-			offset += get_text(title, &buffer[offset]);
-			offset += get_text(description, &buffer[offset]);
+			offset += get_text(title, &event_buffer[offset]);
+			offset += get_text(description, &event_buffer[offset]);
 			
 			if (event.title.is_empty())
 			{
@@ -467,9 +606,10 @@ guint DvbSectionParser::get_bits(const guchar *buffer, guint bitpos, gsize bitco
 	return val;
 }
 
-gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
+
+gsize DvbSectionParser::get_text(String& s, const guchar* text_buffer)
 {
-	gsize length = buffer[0];
+	gsize length = text_buffer[0];
 	guint text_index = 0;
 	gchar text[length];
 	guint index = 0;
@@ -486,9 +626,9 @@ gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
 		}
 		else // Determine codeset from stream
 		{			
-			if (buffer[index] < 0x20)
+			if (text_buffer[index] < 0x20)
 			{
-				switch (buffer[index])
+				switch (text_buffer[index])
 				{
 				case 0x01: codeset = "ISO-8859-5"; break;
 				case 0x02: codeset = "ISO-8859-6"; break;
@@ -507,14 +647,14 @@ gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
 					{
 						// Skip 0x00
 						index++;
-						if (buffer[index] != 0x00)
+						if (text_buffer[index] != 0x00)
 						{
 							Log::write(_("Warning: second byte of code table id was not 0"));
 						}
 						
 						index++;
 						
-						switch(buffer[index])
+						switch(text_buffer[index])
 						{
 						case 0x01: codeset = "ISO-8859-1"; break;
 						case 0x02: codeset = "ISO-8859-2"; break;
@@ -543,7 +683,7 @@ gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
 
 		if (epg_encoding == "iso6937")
 		{
-			s = convert_iso6937(buffer + index, length);
+			s = convert_iso6937(text_buffer + index, length);
 		}
 		else
 		{
@@ -551,7 +691,7 @@ gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
 			{
 				while (index < (length + 1))
 				{
-					u_char ch = buffer[index];
+					u_char ch = text_buffer[index];
 
 					if (codeset == "UTF-16BE")
 					{
@@ -601,14 +741,14 @@ gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
 					Log::write("Length: %d", length);
 					for (guint i = 0; i < (length+1); i++)
 					{
-						gchar ch = buffer[i];
+						gchar ch = text_buffer[i];
 						if (!isprint(ch))
 						{
-							Log::write("buffer[%d]\t= 0x%02X", i, buffer[i]);
+							Log::write("text_buffer[%d]\t= 0x%02X", i, text_buffer[i]);
 						}
 						else
 						{
-							Log::write("buffer[%d]\t= 0x%02X '%c'", i, buffer[i], buffer[i]);
+							Log::write("text_buffer[%d]\t= 0x%02X '%c'", i, text_buffer[i], text_buffer[i]);
 						}
 					}
 					throw Exception(message);
@@ -627,7 +767,7 @@ gsize DvbSectionParser::get_text(String& s, const guchar* buffer)
 	return length + 1;
 }
 
-String DvbSectionParser::convert_iso6937(const guchar* buffer, gsize length)
+String DvbSectionParser::convert_iso6937(const guchar* text_buffer, gsize length)
 {
 	String result;
 	gint val;
@@ -635,19 +775,19 @@ String DvbSectionParser::convert_iso6937(const guchar* buffer, gsize length)
 	
 	for (i=0; i<length; i++ )
 	{
-		if ( buffer[i]<0x20 || (buffer[i]>=0x80 && buffer[i]<=0x9f) )
+		if ( text_buffer[i]<0x20 || (text_buffer[i]>=0x80 && text_buffer[i]<=0x9f) )
 		{
 			continue; // control codes
 		}
 		
-		if ( buffer[i]>=0xC0 && buffer[i]<=0xCF ) // next char has accent
+		if ( text_buffer[i]>=0xC0 && text_buffer[i]<=0xCF ) // next char has accent
 		{
 			if ( i==length-1 ) // Accent char not followed by char
 			{
 				continue;
 			}
 			
-			val = ( buffer[i]<<8 ) + buffer[i+1];
+			val = ( text_buffer[i]<<8 ) + text_buffer[i+1];
 			++i;
 			
 			switch (val)
@@ -822,7 +962,7 @@ String DvbSectionParser::convert_iso6937(const guchar* buffer, gsize length)
 		}
 		else
 		{
-			switch ( buffer[i] )
+			switch ( text_buffer[i] )
 			{
 				case 0xa0: result += gunichar(0x00A0); break; //NO-BREAK SPACE
 				case 0xa1: result += gunichar(0x00A1); break; //INVERTED EXCLAMATION MARK
@@ -897,7 +1037,7 @@ String DvbSectionParser::convert_iso6937(const guchar* buffer, gsize length)
 				case 0xfd:	result += gunichar(0x0167); break; //LATIN SMALL LETTER T WITH STROKE
 				case 0xfe:	result += gunichar(0x014B); break; //LATIN SMALL LETTER ENG (Sami)
 				case 0xff: result += gunichar(0x00AD); break; //SOFT HYPHEN
-				default: result += (char)buffer[i];
+				default: result += (char)text_buffer[i];
 			}
 		}
 	}
