@@ -33,19 +33,9 @@ EpgEvent::EpgEvent()
 
 EpgEvent::EpgEvent(const Channel* channel, const Event& event)
 {
-	struct tm t;
-
-	memset(&t, 0, sizeof(struct tm));
-
-	t.tm_min	= event.start_min;
-	t.tm_hour	= event.start_hour;
-	t.tm_mday	= event.start_day;
-	t.tm_mon	= event.start_month - 1;
-	t.tm_year	= event.start_year - 1900;
-	
 	event_id		= event.event_id;
-	duration		= ((event.dur_hour * 60) + event.dur_min) * 60;
-	start_time		= mktime(&t);
+	duration		= event.duration;
+	start_time		= event.start_time;
 	start_time_text = start_time_to_string(start_time);
 	duration_text	= start_time_to_string(duration);
 	title			= event.title;
@@ -178,7 +168,6 @@ gboolean EpgEvent::operator==(const EpgEvent& e)
 	return e.channel->name == channel->name && e.event_id == event_id;
 }
 
-
 xmlNodePtr Epg::get_channel_node(const Channel& channel)
 {
 	return get_channel_node(channel.name);
@@ -247,6 +236,7 @@ int Epg::add_event(const EpgEvent& event)
 	{
 		glong now = DateTime::now_utc();
 		glong end_time = event.get_start_time() + event.get_duration();
+		
 		if (end_time >= now)
 		{				
 			result++;
