@@ -1,3 +1,23 @@
+/*
+ * Copyright (C) 2008 Michael Lamothe
+ *
+ * This file is part of Me TV
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
+ */
+
 #include "gtk_epg_widget.hh"
 #include "application.hh"
 #include "channel_manager.hh"
@@ -68,16 +88,23 @@ gboolean GtkEpgWidget::is_visible()
 
 void GtkEpgWidget::update()
 {
-	GtkAdjustment* hadjustment = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolled_window_epg));
-	GtkAdjustment* vadjustment = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_window_epg));
+	if (vbox_epg->window != NULL)
+	{
+		gdk_window_freeze_updates(vbox_epg->window);
+							  
+		GtkAdjustment* hadjustment = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(scrolled_window_epg));
+		GtkAdjustment* vadjustment = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scrolled_window_epg));
 	
-	gdouble hvalue = gtk_adjustment_get_value(hadjustment);
-	gdouble vvalue = gtk_adjustment_get_value(vadjustment); 
+		gdouble hvalue = gtk_adjustment_get_value(hadjustment);
+		gdouble vvalue = gtk_adjustment_get_value(vadjustment); 
 
-	update_table();
+		update_table();
 	
-	gtk_adjustment_set_value(hadjustment, hvalue);
-	gtk_adjustment_set_value(vadjustment, vvalue);
+		gtk_adjustment_set_value(hadjustment, hvalue);
+		gtk_adjustment_set_value(vadjustment, vvalue);
+
+		gdk_window_thaw_updates(vbox_epg->window);
+	}
 }
 
 void GtkEpgWidget::update_table()
