@@ -148,7 +148,14 @@ String IO::Channel::read_line()
 
 gboolean IO::Channel::poll(guint timeout)
 {
-	return ::poll(pfd, 1, timeout);
+	guint result = ::poll(pfd, 1, timeout);
+	
+	if (result == -1)
+	{
+		throw SystemException ("Failed to poll");
+	}
+	
+	return result > 0;
 }
 
 gsize IO::Channel::read(gchar* buffer, gsize count)
