@@ -18,40 +18,45 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
  
-#include "main_window.h"
-#include "application.h"
-#include "exception_handler.h"
-#include "scope_log.h"
-#include "xine_engine.h"
-#include "mutex_lock.h"
-#include "gdk_lock.h"
+#include "main_window.hh"
+#include "application.hh"
+#include "exception_handler.hh"
+#include "scope_log.hh"
+#include "xine_engine.hh"
+#include "mutex_lock.hh"
+#include "gdk_lock.hh"
 
 #include <libgnomeui/libgnomeui.h>
 #include <X11/Xlib.h>
 #include <X11/extensions/XTest.h>
 #include "config.h"
 
+#define POKE_INTERVAL				60
+#define CURSOR_HIDE_INTERVAL		3
+
+#define COLUMN_CHANNEL_NAME			0
+#define COLUMN_WEIGHT				1
+
 MainWindow::MainWindow()
 {		
-	engine = NULL;
-
 	Application& application = Application::get_current();
 	Configuration& configuration = application.get_configuration();
 	Glade& glade = application.get_glade();
 	
-	audio_stream_group			= NULL;
-	subtitle_stream_group		= NULL;
+	audio_stream_group		= NULL;
+	subtitle_stream_group	= NULL;
 	
-	window_main					= glade.get_widget("window_main");
-	vpaned_main					= glade.get_widget("vpaned_main");
-	statusbar					= glade.get_widget("statusbar");
-	toolbar						= glade.get_widget("handlebox_toolbar");
-	menubar						= glade.get_widget("menubar");
-	dialog_program_details		= glade.get_widget("dialog_program_details");
-	event_box_video				= glade.get_widget("event_box_video");
+	window_main				= glade.get_widget("window_main");
+	vpaned_main				= glade.get_widget("vpaned_main");
+	statusbar				= glade.get_widget("statusbar");
+	toolbar					= glade.get_widget("handlebox_toolbar");
+	menubar					= glade.get_widget("menubar");
+	dialog_program_details	= glade.get_widget("dialog_program_details");
+	event_box_video			= glade.get_widget("event_box_video");
 	
-	epg_widget = new GtkEpgWidget();
-	preferences_dialog = new PreferencesDialog();
+	epg_widget = NULL;
+	epg_widget					= new GtkEpgWidget();
+	preferences_dialog			= new PreferencesDialog();
 	scheduled_recording_dialog	= new ScheduledRecordingDialog();
 	scheduled_recordings_dialog	= new ScheduledRecordingsDialog();
 	
@@ -87,24 +92,6 @@ MainWindow::MainWindow()
 	g_signal_connect( G_OBJECT ( glade.get_widget("menu_item_audio_dual_language_left") ), "toggled", G_CALLBACK ( on_menu_item_audio_dual_language_changed ), this );
 	g_signal_connect( G_OBJECT ( glade.get_widget("menu_item_audio_dual_language_right") ), "toggled", G_CALLBACK ( on_menu_item_audio_dual_language_changed ), this );
 	
-	GtkWidget* menu_item_subtitles_disabled = glade.get_widget("menu_item_subtitles_disabled");
-	g_signal_connect( G_OBJECT ( menu_item_subtitles_disabled ), "toggled", G_CALLBACK ( on_menu_item_subtitle_stream_toggled ), NULL );
-
-	String engine_type = configuration.get_string_value("engine_type");
-	Log::write(_("Using '%s' engine"), engine_type.c_str());
-	if (engine_type == "xine")
-	{
-		engine = new XineEngine();
-	}
-	else if (engine_type == "none")
-	{
-		// No engine
-	}
-	else
-	{
-		throw Exception(_("Unknown engine type '%s'"), engine_type.c_str());
-	}
-	
 	gint x		= configuration.get_int_value("x");
 	gint y		= configuration.get_int_value("y");
 	gint width	= configuration.get_int_value("width");
@@ -115,11 +102,7 @@ MainWindow::MainWindow()
 		gtk_window_move(GTK_WINDOW(window_main), x, y);
 		gtk_window_resize(GTK_WINDOW(window_main), width, height);
 	}
-
-	gtk_paned_set_position(
-		GTK_PANED(glade.get_widget("vpaned_main")),
-		configuration.get_int_value("video_position"));
-
+	
 	gtk_window_set_keep_above(GTK_WINDOW(window_main), configuration.get_boolean_value("always_on_top"));
 
 	show_controls(configuration.get_boolean_value("show_controls"));
@@ -129,13 +112,14 @@ MainWindow::MainWindow()
 	GSList* iterator = application.get_channel_manager().get_channels();
 	while (iterator != NULL)
 	{
+		GtkTreeIter iter;
 		Channel& channel = *(Channel*)iterator->data;	
 		gtk_combo_box_append_text(combo_box_channel, channel.name.c_str());
 		iterator = g_slist_next(iterator);
 	}
 	gtk_combo_box_set_active(combo_box_channel, 0);
 	gtk_combo_box_set_active(GTK_COMBO_BOX(glade.get_widget("combo_box_start_day")), 0);
-
+	
 	last_motion_time = time(NULL);
 
 	hidden_cursor = NULL;
@@ -145,7 +129,7 @@ MainWindow::MainWindow()
 	GdkPixmap* pixmap = gdk_bitmap_create_from_data(NULL, bits, 1, 1);
 	hidden_cursor = gdk_cursor_new_from_pixmap(pixmap, pixmap, &color, &color, 0, 0);
 
-	is_cursor_visible = true;	
+	is_cursor_visible = true;
 }
 
 MainWindow::~MainWindow()
@@ -153,7 +137,12 @@ MainWindow::~MainWindow()
 	Application& application = Application::get_current();
 	Configuration& configuration = application.get_configuration();
 	
-	configuration.set_int_value("video_position", gtk_paned_get_position(GTK_PANED(vpaned_main)));
+	if (get_epg_visible())
+	{
+		configuration.set_int_value("video_position",
+			gtk_paned_get_position(GTK_PANED(vpaned_main)));
+	}
+	
 	configuration.set_boolean_value("fullscreen", is_fullscreen);
 	configuration.set_boolean_value("show_controls", get_controls_visible());
 	configuration.set_boolean_value("show_epg", get_epg_visible());
@@ -170,12 +159,6 @@ MainWindow::~MainWindow()
 		configuration.set_int_value("width", width);
 		configuration.set_int_value("height", height);
 	}
-
-	if (engine != NULL)
-	{
-		delete engine;
-		engine = NULL;
-	}
 	
 	if (hidden_cursor != NULL)
 	{
@@ -188,6 +171,30 @@ MainWindow::~MainWindow()
 		delete search_bar;
 		search_bar = NULL;
 	}
+	
+	if (scheduled_recordings_dialog != NULL)
+	{
+		delete scheduled_recordings_dialog;
+		scheduled_recordings_dialog = NULL;
+	}
+
+	if (scheduled_recording_dialog != NULL)
+	{
+		delete scheduled_recording_dialog;
+		scheduled_recording_dialog = NULL;
+	}
+	
+	if (preferences_dialog != NULL)
+	{
+		delete preferences_dialog;
+		preferences_dialog = NULL;
+	}
+	
+	if (epg_widget != NULL)
+	{
+		delete epg_widget;
+		epg_widget = NULL;
+	}
 }
 
 gboolean MainWindow::on_window_main_delete (GtkWindow* window_main, GdkEvent* event, MainWindow* main_window)
@@ -198,14 +205,16 @@ gboolean MainWindow::on_window_main_delete (GtkWindow* window_main, GdkEvent* ev
 	gboolean quit_on_close = configuration.get_boolean_value("quit_on_close");
 	gboolean has_scheduled_recordings = application.get_recording_manager().has_scheduled_recordings();
 	
+	main_window->hide();
 	if (quit_on_close && !has_scheduled_recordings)
 	{
 		application.quit();
 	}
 	else
 	{
-		main_window->hide();
+		application.mute(true);
 	}
+
 	return TRUE;
 }
 
@@ -235,6 +244,7 @@ void MainWindow::show_epg(gboolean show)
 {
 	if (show)
 	{
+		update_epg();
 		epg_widget->show();
 	}
 	else
@@ -244,16 +254,16 @@ void MainWindow::show_epg(gboolean show)
 	}
 
 	Glade& glade = Application::get_current().get_glade();
-	GtkCheckMenuItem* menu_item_program_guide = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_view_epg"));
-	gboolean active = gtk_check_menu_item_get_active(menu_item_program_guide);
+	GtkCheckMenuItem* menu_item_view_epg = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_view_epg"));
+	gboolean active = gtk_check_menu_item_get_active(menu_item_view_epg);
 	if (active != show)
 	{
-		gtk_check_menu_item_set_active(menu_item_program_guide, show);
+		gtk_check_menu_item_set_active(menu_item_view_epg, show);
 	}
 }
 
 void MainWindow::show_fullscreen(gboolean show)
-{	
+{
 	if (show)
 	{
 		gtk_window_fullscreen(GTK_WINDOW(window_main));
@@ -266,7 +276,7 @@ void MainWindow::show_fullscreen(gboolean show)
 	is_fullscreen = show;
 }
 
-void MainWindow::toggle_controls()
+void MainWindow::toggle_epg()
 {	
 	if (is_fullscreen)
 	{
@@ -403,7 +413,7 @@ void MainWindow::on_menu_item_view_fullscreen_activate (GtkMenuItem* widget, Mai
 
 void MainWindow::on_menu_item_view_epg_activate (GtkCheckMenuItem* widget, MainWindow* main_window)
 {
-	TRY
+	TRY	
 	main_window->show_epg(gtk_check_menu_item_get_active(widget));
 	CATCH
 }
@@ -420,11 +430,33 @@ void MainWindow::on_menu_item_audio_stream_toggled(GtkCheckMenuItem *checkmenuit
 	TRY
 	if (gtk_check_menu_item_get_active(checkmenuitem))
 	{
-		gint audio_pid = *((gint *)data);
+		guint audio_pid = (guint)(glong)data;
 		Log::write(_("Setting audio PID to %d"), audio_pid);
 		Application& application = Application::get_current();
-		application.get_video_channel().default_audio_pid = audio_pid;
-		application.restart_stream();
+		Configuration& configuration = application.get_configuration();
+
+		Engine* engine = application.get_engine();
+		if (engine != NULL)
+		{
+			if (audio_pid == 0)
+			{
+				engine->set_audio_channel(-1);
+				configuration.set_string_value("preferred_language", "");
+			}
+			else
+			{
+				const Stream& stream = application.get_streamer().get_stream();
+				for (guint i = 0; i < stream.audio_streams.size(); i++)
+				{
+					AudioStream audio_stream = stream.audio_streams[i];
+					if (audio_stream.pid == audio_pid)
+					{
+						engine->set_audio_channel(i);
+						configuration.set_string_value("preferred_language", audio_stream.language);
+					}
+				}
+			}
+		}
 	}
 	CATCH
 }
@@ -434,17 +466,33 @@ void MainWindow::on_menu_item_subtitle_stream_toggled(GtkCheckMenuItem *checkmen
 	TRY
 	if (gtk_check_menu_item_get_active(checkmenuitem))
 	{
-		gint subtitle_pid = 0;
-		
-		if (data != NULL)
-		{
-			subtitle_pid = *((gint *)data);
-		}
-		
+		guint subtitle_pid = (guint)(glong)data;		
 		Log::write(_("Setting subtitle PID to %d"), subtitle_pid);
 		Application& application = Application::get_current();
-		application.get_video_channel().default_subtitle_pid = subtitle_pid;
-		application.restart_stream();
+		Configuration& configuration = application.get_configuration();
+
+		Engine* engine = application.get_engine();
+		if (engine != NULL)
+		{
+			if (subtitle_pid == 0)
+			{
+				engine->set_subtitle_channel(-1);
+				configuration.set_string_value("preferred_language", "");
+			}
+			else
+			{
+				const Stream& stream = application.get_streamer().get_stream();
+				for (guint i = 0; i < stream.subtitle_streams.size(); i++)
+				{
+					SubtitleStream subtitle_stream = stream.subtitle_streams[i];
+					if (subtitle_stream.pid == subtitle_pid)
+					{
+						engine->set_subtitle_channel(i);
+						configuration.set_string_value("preferred_language", subtitle_stream.language);
+					}
+				}
+			}
+		}
 	}
 	CATCH
 }
@@ -490,7 +538,7 @@ gboolean MainWindow::on_window_main_key_press_event(GtkWidget* window_main, GdkE
 			break;
 		case GDK_e:
 		case GDK_E:
-			Application::get_current().toggle_controls();
+			Application::get_current().toggle_epg();
 			break;
 		case GDK_f:
 		case GDK_F:
@@ -515,6 +563,61 @@ gboolean MainWindow::on_window_main_key_press_event(GtkWidget* window_main, GdkE
 		case GDK_H:
 			Application::get_current().hide_controls();
 			break;
+		case GDK_Down:
+		case GDK_Page_Down:
+			{
+				Application& application = Application::get_current();
+				String video_channel_name = application.get_video_channel().name;
+				GSList* channels = application.get_channel_manager().get_channels();
+				gboolean done = (channels == NULL);
+				while(!done)
+				{
+					Channel* channel = (Channel*)channels->data;
+					channels = g_slist_next(channels);
+
+					if (channels != NULL)
+					{
+						if (channel->name == video_channel_name)
+						{
+							Channel* next_channel = (Channel*)channels->data;
+							if (next_channel != NULL)
+							{
+								application.change_channel(*next_channel);
+							}
+							done = true;
+						}
+					}
+					done |= (channels == NULL);
+				}
+			}
+			break;
+		case GDK_Up:
+		case GDK_Page_Up:
+			{
+				Application& application = Application::get_current();
+				String video_channel_name = application.get_video_channel().name;
+				GSList* channels = application.get_channel_manager().get_channels();
+				Channel* prev_channel = NULL;
+				gboolean done = (channels == NULL);
+				while(!done)
+				{
+					Channel* channel = (Channel*)channels->data;
+
+					if (prev_channel != NULL)
+					{
+						if (channel->name == video_channel_name)
+						{
+							application.change_channel(*prev_channel);
+							done = true;
+						}
+					}
+
+					prev_channel = channel;
+					channels = g_slist_next(channels);
+					done |= (channels == NULL);
+				}
+			}
+			break;
 	}
 	CATCH
 	
@@ -532,9 +635,9 @@ gboolean MainWindow::on_window_main_button_press(GtkWindow* widget, GdkEventButt
 			application.toggle_fullscreen();
 		}
 	}
-	else if (event->button == 2 || event->button == 3)
+	else if (event->button == 3)
 	{
-		application.toggle_controls();
+		application.toggle_epg();
 	}
 	CATCH
 
@@ -591,24 +694,56 @@ void MainWindow::on_spinbutton_duration_value_changed (GtkSpinButton* widget, Ma
 	CATCH
 }
 
+guint MainWindow::get_dual_language_state()
+{
+	Glade& glade = Application::get_current().get_glade();
+	guint result = ENGINE_DUAL_LANGUAGE_STATE_DISABLED;
+
+	GtkCheckMenuItem* menu_item_audio_dual_language_left = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_audio_dual_language_left"));
+	GtkCheckMenuItem* menu_item_audio_dual_language_right = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_audio_dual_language_right"));
+	
+	if (gtk_check_menu_item_get_active(menu_item_audio_dual_language_left))
+	{
+		result = ENGINE_DUAL_LANGUAGE_STATE_LEFT;
+	}
+	else if (gtk_check_menu_item_get_active(menu_item_audio_dual_language_right))
+	{
+		result = ENGINE_DUAL_LANGUAGE_STATE_RIGHT;
+	}
+	
+	return result;
+}
+
+void MainWindow::set_dual_language_state(guint dual_language_state)
+{	
+	if (dual_language_state != get_dual_language_state ())
+	{
+		Glade& glade = Application::get_current().get_glade();
+		GtkCheckMenuItem* check_menu_item = NULL;
+		
+		switch(dual_language_state)
+		{
+			case ENGINE_DUAL_LANGUAGE_STATE_LEFT:
+				check_menu_item = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_audio_dual_language_left"));
+				break;
+			case ENGINE_DUAL_LANGUAGE_STATE_RIGHT:
+				check_menu_item = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_audio_dual_language_right"));
+				break;
+			default:
+				check_menu_item = GTK_CHECK_MENU_ITEM(glade.get_widget("menu_item_audio_dual_language_disable"));
+				break;
+		}
+		
+		gtk_check_menu_item_set_active(check_menu_item, true);
+	}
+}
+
 void MainWindow::on_menu_item_audio_dual_language_changed(GtkCheckMenuItem* widget, MainWindow* main_window)
 {
 	TRY
 	if (gtk_check_menu_item_get_active(widget))
 	{
-		GtkWidget* w = GTK_WIDGET(widget);
-		gint state = ENGINE_DUAL_LANGUAGE_DISABLE;
-		Glade& glade = Application::get_current().get_glade();
-		if (w == glade.get_widget("menu_item_audio_dual_language_left"))
-		{
-			state = ENGINE_DUAL_LANGUAGE_LEFT;
-		}
-		else if (w == glade.get_widget("menu_item_audio_dual_language_right"))
-		{
-			state = ENGINE_DUAL_LANGUAGE_RIGHT;
-		}
-		
-		main_window->engine->set_dual_language_state(state);
+		Application::get_current().set_dual_language_state(main_window->get_dual_language_state());
 	}
 	CATCH
 }
@@ -623,7 +758,9 @@ void MainWindow::update_audio_stream_menu()
 {
 	Application& application = Application::get_current();
 	Glade& glade = application.get_glade();
-	Channel& channel = application.get_video_channel();
+	Streamer& streamer = application.get_streamer();
+	const Stream& stream = streamer.get_stream();
+	gboolean channel_set = false;
 	
 	GtkWidget* menuitem_audio_streams = glade.get_widget("menu_item_audio_streams");
 	GtkWidget* menu_widget = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menuitem_audio_streams));
@@ -633,30 +770,27 @@ void MainWindow::update_audio_stream_menu()
 		gtk_widget_destroy(menu_widget);
 	}
 
-	guint audio_pid_size = channel.audio_pids.size();
-	if (audio_pid_size > 0)
-	{
-		if (audio_pid_size > MAX_AUDIO_PIDS)
-		{
-			audio_pid_size = MAX_AUDIO_PIDS;
-		}
-		
+	guint audio_stream_size = stream.audio_streams.size();
+	if (audio_stream_size > 0)
+	{		
+		String preferred_language = application.get_configuration().get_string_value("preferred_language");
+
 		menu_widget = gtk_menu_new();
-		for (guint i = 0; i < audio_pid_size; i++)
+		for (guint i = 0; i < audio_stream_size; i++)
 		{
-			guint audio_pid = channel.audio_pids[i];
-			audio_pids[i] = audio_pid;
-			String label = String::format(_("Audio Stream %d (PID %d)"), i + 1, audio_pid);
+			AudioStream audio_stream = stream.audio_streams[i];
+			String label = String::format("%d: '%s' (PID %d)", i + 1, audio_stream.language.c_str(), audio_stream.pid);
 			GtkWidget* menu_item = gtk_radio_menu_item_new_with_label(audio_stream_group, label.c_str());
-			audio_stream_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
-			gtk_menu_shell_append(GTK_MENU_SHELL(menu_widget), menu_item);
 			
-			if (channel.default_audio_pid == audio_pid)
+			g_signal_connect( G_OBJECT ( menu_item ), "toggled", G_CALLBACK ( on_menu_item_audio_stream_toggled ), (gpointer)audio_stream.pid );
+			if (!channel_set && preferred_language == audio_stream.language)
 			{
+				channel_set = true;
 				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), true);
 			}
-			
-			g_signal_connect( G_OBJECT ( menu_item ), "toggled", G_CALLBACK ( on_menu_item_audio_stream_toggled ), &(audio_pids[i]) );
+
+			audio_stream_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu_widget), menu_item);
 			gtk_widget_show(menu_item);
 		}
 		gtk_widget_show(menu_widget);
@@ -669,37 +803,49 @@ void MainWindow::update_view_subtitle_menu()
 {
 	Application& application = Application::get_current();
 	Glade& glade = application.get_glade();
-	Channel& channel = application.get_video_channel();
-	
+	Streamer& streamer = application.get_streamer();
+	const Stream& stream = streamer.get_stream();
+	gboolean channel_set = false;
+
 	GtkWidget* menu_item_view_subtitles = glade.get_widget("menu_item_view_subtitles");
 	GtkWidget* menu_widget = gtk_menu_item_get_submenu(GTK_MENU_ITEM(menu_item_view_subtitles));
-		
-	guint subtitle_pid_size = channel.subtitle_pids.size();
-	if (subtitle_pid_size > 0)
+	
+	if (menu_widget != NULL)
+	{
+		gtk_widget_destroy(menu_widget);
+		subtitle_stream_group = NULL;
+	}
+	
+	guint subtitle_stream_size = stream.subtitle_streams.size();
+	if (subtitle_stream_size > 0)
 	{
-		if (subtitle_pid_size > MAX_AUDIO_PIDS)
-		{
-			subtitle_pid_size = MAX_AUDIO_PIDS;
-		}
-		
 		menu_widget = gtk_menu_new();
-		for (guint i = 0; i < subtitle_pid_size; i++)
+		
+		GtkWidget* disabled_menu_item = gtk_radio_menu_item_new_with_label(subtitle_stream_group, _("Subtitles disabled"));
+		subtitle_stream_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (disabled_menu_item));
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu_widget), disabled_menu_item);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(disabled_menu_item), true);
+		g_signal_connect( G_OBJECT ( disabled_menu_item ), "toggled", G_CALLBACK ( on_menu_item_subtitle_stream_toggled ), (gpointer)0 );
+		gtk_widget_show(disabled_menu_item);
+
+		String preferred_language = application.get_configuration().get_string_value("preferred_language");
+		for (guint i = 0; i < subtitle_stream_size; i++)
 		{
-			guint subtitle_pid = channel.subtitle_pids[i];
-			subtitle_pids[i] = subtitle_pid;
-			String label = String::format(_("Subtitle Stream %d (PID %d)"), i + 1, subtitle_pid);
+			SubtitleStream subtitle_stream = stream.subtitle_streams[i];
+			String label = String::format("%d: %s (PID %d)", i + 1, subtitle_stream.language.c_str(), subtitle_stream.pid);
 			GtkWidget* menu_item = gtk_radio_menu_item_new_with_label(subtitle_stream_group, label.c_str());
 			subtitle_stream_group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menu_item));
 			gtk_menu_shell_append(GTK_MENU_SHELL(menu_widget), menu_item);
 			
-			if (channel.default_subtitle_pid == subtitle_pid)
+			g_signal_connect( G_OBJECT ( menu_item ), "toggled", G_CALLBACK ( on_menu_item_subtitle_stream_toggled ), (gpointer)subtitle_stream.pid );
+			if (!channel_set && preferred_language == subtitle_stream.language)
 			{
+				channel_set = true;
 				gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), true);
 			}
-			
-			g_signal_connect( G_OBJECT ( menu_item ), "toggled", G_CALLBACK ( on_menu_item_subtitle_stream_toggled ), &(subtitle_pids[i]) );
 			gtk_widget_show(menu_item);
 		}
+		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item_view_subtitles), menu_widget);
 		gtk_widget_show(menu_widget);
 	}
 }
@@ -737,11 +883,6 @@ void MainWindow::record(gboolean state)
 
 void MainWindow::mute(gboolean state)
 {
-	if (engine != NULL)
-	{
-		engine->mute(state);
-	}
-	
 	Glade& glade = Application::get_current().get_glade();
 	
 	GtkToggleToolButton* tool_button_mute = GTK_TOGGLE_TOOL_BUTTON(glade.get_widget("tool_button_mute"));
@@ -761,6 +902,7 @@ void MainWindow::auto_surf(gboolean state)
 {
 	GtkToggleToolButton* tool_button_auto_surf = GTK_TOGGLE_TOOL_BUTTON(
 		Application::get_current().get_glade().get_widget("tool_button_auto_surf"));
+	
 	if (gtk_toggle_tool_button_get_active(tool_button_auto_surf) != state)
 	{
 		gtk_toggle_tool_button_set_active(tool_button_auto_surf, state);
@@ -777,25 +919,12 @@ void MainWindow::broadcast(gboolean state)
 	}
 }
 
-void MainWindow::subtitle_channel(int channel)
-{
-	if (engine != NULL)
-	{
-		engine->set_subtitle_channel(channel);
-	}
-}
-
-void MainWindow::audio_channel(int channel)
-{
-	if (engine != NULL)
-	{
-		engine->set_audio_channel(channel);
-	}
-}
-
 void MainWindow::update_epg()
 {
-	epg_widget->update();
+	if (get_epg_visible())
+	{
+		epg_widget->update();
+	}
 	
 	Application& application = Application::get_current();
 	String message = application.get_status_message();
@@ -806,14 +935,22 @@ void MainWindow::update_epg()
 
 void MainWindow::on_timer()
 {
-	if (time(NULL) - last_motion_time > 3)
+	static time_t last_poke_time = 0;
+
+	time_t now = DateTime::now();
+
+	if (now - last_motion_time > CURSOR_HIDE_INTERVAL)
 	{
 		hide_cursor();
 	}
 	
-	if (engine != NULL)
+	if (now - last_poke_time > POKE_INTERVAL)
 	{
-		engine->expose();
+		if (is_visible() && !is_minimised()) 		
+		{ 		
+			send_fake_key();
+		}
+		last_poke_time = now;
 	}
 }
 
@@ -837,18 +974,28 @@ void MainWindow::show()
 	gint y = configuration.get_int_value("y");
 
 	gtk_window_move(GTK_WINDOW(window_main), x, y);
+
+	gtk_paned_set_position(
+		GTK_PANED(vpaned_main),
+		configuration.get_int_value("video_position"));	
 }
 
 void MainWindow::hide()
 {
 	gint x = 0;
 	gint y = 0;
-
-	mute(true);
+	
 	Configuration& configuration = Application::get_current().get_configuration();
 	gtk_window_get_position(GTK_WINDOW(window_main), &x, &y);
 	configuration.set_int_value("x", x);
 	configuration.set_int_value("y", y);
+	
+	if (get_epg_visible())
+	{
+		configuration.set_int_value("video_position",
+			gtk_paned_get_position(GTK_PANED(vpaned_main)));
+	}
+	
 	gtk_widget_hide(GTK_WIDGET(window_main));
 }
 
@@ -1015,5 +1162,4 @@ gboolean MainWindow::is_minimised()
 { 		
 	GdkWindowState state = gdk_window_get_state(GTK_WIDGET(window_main)->window); 		
 	return state & GDK_WINDOW_STATE_ICONIFIED;
-	return false;
 }
