@@ -18,30 +18,30 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
 
-#include "network.h"
-#include "exception.h"
-#include <libgnome/libgnome.h>
+#include "network.hh"
+#include "exception.hh"
+#include "application.hh"
 
-MulticastSocket::MulticastSocket(const gchar* hostname, int port)
+UdpSocket::UdpSocket(const gchar* hostname, int port)
 {
 	address = gnet_inetaddr_new(hostname, port);
-	socket = gnet_mcast_socket_new_with_port(port);
+	socket = gnet_udp_socket_new();
 
 	if (socket == NULL)
 	{
-		throw Exception(_("Failed to create a multicast socket"));
+		throw Exception(_("Failed to create a UDP socket"));
 	}
 }
 
-MulticastSocket::~MulticastSocket()
+UdpSocket::~UdpSocket()
 {
-	gnet_mcast_socket_delete(socket);
+	gnet_udp_socket_delete(socket);
 }
 
-void MulticastSocket::write(const void* buffer, unsigned int length)
+void UdpSocket::write(const void* buffer, unsigned int length)
 {
-	if (gnet_mcast_socket_send(socket, (const gchar*)buffer, length, address) != 0)
+	if (gnet_udp_socket_send(socket, (const gchar*)buffer, length, address) != 0)
 	{
-		throw SystemException("Failed to write UDP packet");
+		throw SystemException(_("Failed to write UDP packet"));
 	}
 }
