@@ -28,7 +28,7 @@
 #define COLUMN_REGION	0
 
 #ifndef SCAN_DIRECTORY
-#define SCAN_DIRECTORY "/usr/share/doc/dvb-utils/examples/scan"
+#define SCAN_DIRECTORY "/usr/share/dvb"
 #endif
 
 class Timeout
