@@ -1,4 +1,4 @@
-#include "scheduled_recording.h"
+#include "scheduled_recording.hh"
 
 ScheduledRecording::ScheduledRecording()
 {
